
'Generated protocol buffer code.'
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/python/util/protobuf/compare_test.proto', package='compare_test', syntax='proto2', serialized_options=b'\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n2tensorflow/python/util/protobuf/compare_test.proto\x12\x0ccompare_test"\x18\n\x05Small\x12\x0f\n\x07strings\x18\x01 \x03(\t"\xd3\x01\n\x06Medium\x12\x0e\n\x06int32s\x18\x01 \x03(\x05\x12#\n\x06smalls\x18\x02 \x03(\x0b2\x13.compare_test.Small\x12+\n\x06groupa\x18\x03 \x03(\n2\x1b.compare_test.Medium.GroupA\x12\x0e\n\x06floats\x18\x06 \x03(\x02\x1aW\n\x06GroupA\x122\n\x06groupb\x18\x04 \x03(\n2".compare_test.Medium.GroupA.GroupB\x1a\x19\n\x06GroupB\x12\x0f\n\x07strings\x18\x05 \x02(\t"\xfe\x01\n\x05Large\x12\x0f\n\x07string_\x18\x01 \x01(\t\x12\x0e\n\x06int64_\x18\x02 \x01(\x03\x12\x0e\n\x06float_\x18\x03 \x01(\x02\x12\r\n\x05bool_\x18\x04 \x01(\x08\x12!\n\x05enum_\x18\x05 \x01(\x0e2\x12.compare_test.Enum\x12\x0e\n\x06int64s\x18\x06 \x03(\x03\x12$\n\x06medium\x18\x07 \x01(\x0b2\x14.compare_test.Medium\x12"\n\x05small\x18\x08 \x01(\x0b2\x13.compare_test.Small\x12\x0f\n\x07double_\x18\t \x01(\x01\x12\'\n\x08with_map\x18\n \x01(\x0b2\x15.compare_test.WithMap"-\n\x07Labeled\x12\x10\n\x08required\x18\x01 \x02(\x05\x12\x10\n\x08optional\x18\x02 \x01(\x05"\x85\x02\n\x07WithMap\x12>\n\rvalue_message\x18\x01 \x03(\x0b2\'.compare_test.WithMap.ValueMessageEntry\x12<\n\x0cvalue_string\x18\x02 \x03(\x0b2&.compare_test.WithMap.ValueStringEntry\x1aH\n\x11ValueMessageEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12"\n\x05value\x18\x02 \x01(\x0b2\x13.compare_test.Small:\x028\x01\x1a2\n\x10ValueStringEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x028\x01*\x1b\n\x04Enum\x12\x05\n\x01A\x10\x00\x12\x05\n\x01B\x10\x01\x12\x05\n\x01C\x10\x02B\x03\xf8\x01\x01')
_ENUM = _descriptor.EnumDescriptor(name='Enum', full_name='compare_test.Enum', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='A', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='B', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='C', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=876, serialized_end=903)
_sym_db.RegisterEnumDescriptor(_ENUM)
Enum = enum_type_wrapper.EnumTypeWrapper(_ENUM)
A = 0
B = 1
C = 2
_SMALL = _descriptor.Descriptor(name='Small', full_name='compare_test.Small', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='strings', full_name='compare_test.Small.strings', index=0, number=1, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=68, serialized_end=92)
_MEDIUM_GROUPA_GROUPB = _descriptor.Descriptor(name='GroupB', full_name='compare_test.Medium.GroupA.GroupB', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='strings', full_name='compare_test.Medium.GroupA.GroupB.strings', index=0, number=5, type=9, cpp_type=9, label=2, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=281, serialized_end=306)
_MEDIUM_GROUPA = _descriptor.Descriptor(name='GroupA', full_name='compare_test.Medium.GroupA', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='groupb', full_name='compare_test.Medium.GroupA.groupb', index=0, number=4, type=10, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_MEDIUM_GROUPA_GROUPB], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=219, serialized_end=306)
_MEDIUM = _descriptor.Descriptor(name='Medium', full_name='compare_test.Medium', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='int32s', full_name='compare_test.Medium.int32s', index=0, number=1, type=5, cpp_type=1, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='smalls', full_name='compare_test.Medium.smalls', index=1, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='groupa', full_name='compare_test.Medium.groupa', index=2, number=3, type=10, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='floats', full_name='compare_test.Medium.floats', index=3, number=6, type=2, cpp_type=6, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_MEDIUM_GROUPA], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=95, serialized_end=306)
_LARGE = _descriptor.Descriptor(name='Large', full_name='compare_test.Large', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='string_', full_name='compare_test.Large.string_', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='int64_', full_name='compare_test.Large.int64_', index=1, number=2, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='float_', full_name='compare_test.Large.float_', index=2, number=3, type=2, cpp_type=6, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='bool_', full_name='compare_test.Large.bool_', index=3, number=4, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='enum_', full_name='compare_test.Large.enum_', index=4, number=5, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='int64s', full_name='compare_test.Large.int64s', index=5, number=6, type=3, cpp_type=2, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='medium', full_name='compare_test.Large.medium', index=6, number=7, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='small', full_name='compare_test.Large.small', index=7, number=8, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='double_', full_name='compare_test.Large.double_', index=8, number=9, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='with_map', full_name='compare_test.Large.with_map', index=9, number=10, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=309, serialized_end=563)
_LABELED = _descriptor.Descriptor(name='Labeled', full_name='compare_test.Labeled', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='required', full_name='compare_test.Labeled.required', index=0, number=1, type=5, cpp_type=1, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='optional', full_name='compare_test.Labeled.optional', index=1, number=2, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=565, serialized_end=610)
_WITHMAP_VALUEMESSAGEENTRY = _descriptor.Descriptor(name='ValueMessageEntry', full_name='compare_test.WithMap.ValueMessageEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='compare_test.WithMap.ValueMessageEntry.key', index=0, number=1, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='compare_test.WithMap.ValueMessageEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=750, serialized_end=822)
_WITHMAP_VALUESTRINGENTRY = _descriptor.Descriptor(name='ValueStringEntry', full_name='compare_test.WithMap.ValueStringEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='compare_test.WithMap.ValueStringEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='compare_test.WithMap.ValueStringEntry.value', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=824, serialized_end=874)
_WITHMAP = _descriptor.Descriptor(name='WithMap', full_name='compare_test.WithMap', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='value_message', full_name='compare_test.WithMap.value_message', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value_string', full_name='compare_test.WithMap.value_string', index=1, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_WITHMAP_VALUEMESSAGEENTRY, _WITHMAP_VALUESTRINGENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto2', extension_ranges=[], oneofs=[], serialized_start=613, serialized_end=874)
_MEDIUM_GROUPA_GROUPB.containing_type = _MEDIUM_GROUPA
_MEDIUM_GROUPA.fields_by_name['groupb'].message_type = _MEDIUM_GROUPA_GROUPB
_MEDIUM_GROUPA.containing_type = _MEDIUM
_MEDIUM.fields_by_name['smalls'].message_type = _SMALL
_MEDIUM.fields_by_name['groupa'].message_type = _MEDIUM_GROUPA
_LARGE.fields_by_name['enum_'].enum_type = _ENUM
_LARGE.fields_by_name['medium'].message_type = _MEDIUM
_LARGE.fields_by_name['small'].message_type = _SMALL
_LARGE.fields_by_name['with_map'].message_type = _WITHMAP
_WITHMAP_VALUEMESSAGEENTRY.fields_by_name['value'].message_type = _SMALL
_WITHMAP_VALUEMESSAGEENTRY.containing_type = _WITHMAP
_WITHMAP_VALUESTRINGENTRY.containing_type = _WITHMAP
_WITHMAP.fields_by_name['value_message'].message_type = _WITHMAP_VALUEMESSAGEENTRY
_WITHMAP.fields_by_name['value_string'].message_type = _WITHMAP_VALUESTRINGENTRY
DESCRIPTOR.message_types_by_name['Small'] = _SMALL
DESCRIPTOR.message_types_by_name['Medium'] = _MEDIUM
DESCRIPTOR.message_types_by_name['Large'] = _LARGE
DESCRIPTOR.message_types_by_name['Labeled'] = _LABELED
DESCRIPTOR.message_types_by_name['WithMap'] = _WITHMAP
DESCRIPTOR.enum_types_by_name['Enum'] = _ENUM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
Small = _reflection.GeneratedProtocolMessageType('Small', (_message.Message,), {'DESCRIPTOR': _SMALL, '__module__': 'tensorflow.python.util.protobuf.compare_test_pb2'})
_sym_db.RegisterMessage(Small)
Medium = _reflection.GeneratedProtocolMessageType('Medium', (_message.Message,), {'GroupA': _reflection.GeneratedProtocolMessageType('GroupA', (_message.Message,), {'GroupB': _reflection.GeneratedProtocolMessageType('GroupB', (_message.Message,), {'DESCRIPTOR': _MEDIUM_GROUPA_GROUPB, '__module__': 'tensorflow.python.util.protobuf.compare_test_pb2'}), 'DESCRIPTOR': _MEDIUM_GROUPA, '__module__': 'tensorflow.python.util.protobuf.compare_test_pb2'}), 'DESCRIPTOR': _MEDIUM, '__module__': 'tensorflow.python.util.protobuf.compare_test_pb2'})
_sym_db.RegisterMessage(Medium)
_sym_db.RegisterMessage(Medium.GroupA)
_sym_db.RegisterMessage(Medium.GroupA.GroupB)
Large = _reflection.GeneratedProtocolMessageType('Large', (_message.Message,), {'DESCRIPTOR': _LARGE, '__module__': 'tensorflow.python.util.protobuf.compare_test_pb2'})
_sym_db.RegisterMessage(Large)
Labeled = _reflection.GeneratedProtocolMessageType('Labeled', (_message.Message,), {'DESCRIPTOR': _LABELED, '__module__': 'tensorflow.python.util.protobuf.compare_test_pb2'})
_sym_db.RegisterMessage(Labeled)
WithMap = _reflection.GeneratedProtocolMessageType('WithMap', (_message.Message,), {'ValueMessageEntry': _reflection.GeneratedProtocolMessageType('ValueMessageEntry', (_message.Message,), {'DESCRIPTOR': _WITHMAP_VALUEMESSAGEENTRY, '__module__': 'tensorflow.python.util.protobuf.compare_test_pb2'}), 'ValueStringEntry': _reflection.GeneratedProtocolMessageType('ValueStringEntry', (_message.Message,), {'DESCRIPTOR': _WITHMAP_VALUESTRINGENTRY, '__module__': 'tensorflow.python.util.protobuf.compare_test_pb2'}), 'DESCRIPTOR': _WITHMAP, '__module__': 'tensorflow.python.util.protobuf.compare_test_pb2'})
_sym_db.RegisterMessage(WithMap)
_sym_db.RegisterMessage(WithMap.ValueMessageEntry)
_sym_db.RegisterMessage(WithMap.ValueStringEntry)
DESCRIPTOR._options = None
_WITHMAP_VALUEMESSAGEENTRY._options = None
_WITHMAP_VALUESTRINGENTRY._options = None




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from ....tensorflow.core.framework import graph_pb2 as tensorflow_dot_core_dot_framework_dot_graph__pb2
from ....tensorflow.core.framework import op_def_pb2 as tensorflow_dot_core_dot_framework_dot_op__def__pb2
from ....tensorflow.core.framework import tensor_shape_pb2 as tensorflow_dot_core_dot_framework_dot_tensor__shape__pb2
from ....tensorflow.core.framework import types_pb2 as tensorflow_dot_core_dot_framework_dot_types__pb2
from ....tensorflow.core.protobuf import saved_object_graph_pb2 as tensorflow_dot_core_dot_protobuf_dot_saved__object__graph__pb2
from ....tensorflow.core.protobuf import saver_pb2 as tensorflow_dot_core_dot_protobuf_dot_saver__pb2
from ....tensorflow.core.protobuf import struct_pb2 as tensorflow_dot_core_dot_protobuf_dot_struct__pb2
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/protobuf/meta_graph.proto', package='tensorflow', syntax='proto3', serialized_options=b'\n\x18org.tensorflow.frameworkB\x0fMetaGraphProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n)tensorflow/core/protobuf/meta_graph.proto\x12\ntensorflow\x1a\x19google/protobuf/any.proto\x1a%tensorflow/core/framework/graph.proto\x1a&tensorflow/core/framework/op_def.proto\x1a,tensorflow/core/framework/tensor_shape.proto\x1a%tensorflow/core/framework/types.proto\x1a1tensorflow/core/protobuf/saved_object_graph.proto\x1a$tensorflow/core/protobuf/saver.proto\x1a%tensorflow/core/protobuf/struct.proto"\xa8\x07\n\x0cMetaGraphDef\x12;\n\rmeta_info_def\x18\x01 \x01(\x0b2$.tensorflow.MetaGraphDef.MetaInfoDef\x12\'\n\tgraph_def\x18\x02 \x01(\x0b2\x14.tensorflow.GraphDef\x12\'\n\tsaver_def\x18\x03 \x01(\x0b2\x14.tensorflow.SaverDef\x12C\n\x0ecollection_def\x18\x04 \x03(\x0b2+.tensorflow.MetaGraphDef.CollectionDefEntry\x12A\n\rsignature_def\x18\x05 \x03(\x0b2*.tensorflow.MetaGraphDef.SignatureDefEntry\x120\n\x0easset_file_def\x18\x06 \x03(\x0b2\x18.tensorflow.AssetFileDef\x126\n\x10object_graph_def\x18\x07 \x01(\x0b2\x1c.tensorflow.SavedObjectGraph\x1a\xf6\x02\n\x0bMetaInfoDef\x12\x1a\n\x12meta_graph_version\x18\x01 \x01(\t\x12,\n\x10stripped_op_list\x18\x02 \x01(\x0b2\x12.tensorflow.OpList\x12&\n\x08any_info\x18\x03 \x01(\x0b2\x14.google.protobuf.Any\x12\x0c\n\x04tags\x18\x04 \x03(\t\x12\x1a\n\x12tensorflow_version\x18\x05 \x01(\t\x12\x1e\n\x16tensorflow_git_version\x18\x06 \x01(\t\x12\x1e\n\x16stripped_default_attrs\x18\x07 \x01(\x08\x12S\n\x10function_aliases\x18\x08 \x03(\x0b29.tensorflow.MetaGraphDef.MetaInfoDef.FunctionAliasesEntry\x1a6\n\x14FunctionAliasesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x028\x01\x1aO\n\x12CollectionDefEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b2\x19.tensorflow.CollectionDef:\x028\x01\x1aM\n\x11SignatureDefEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\'\n\x05value\x18\x02 \x01(\x0b2\x18.tensorflow.SignatureDef:\x028\x01"\xdf\x03\n\rCollectionDef\x127\n\tnode_list\x18\x01 \x01(\x0b2".tensorflow.CollectionDef.NodeListH\x00\x129\n\nbytes_list\x18\x02 \x01(\x0b2#.tensorflow.CollectionDef.BytesListH\x00\x129\n\nint64_list\x18\x03 \x01(\x0b2#.tensorflow.CollectionDef.Int64ListH\x00\x129\n\nfloat_list\x18\x04 \x01(\x0b2#.tensorflow.CollectionDef.FloatListH\x00\x125\n\x08any_list\x18\x05 \x01(\x0b2!.tensorflow.CollectionDef.AnyListH\x00\x1a\x19\n\x08NodeList\x12\r\n\x05value\x18\x01 \x03(\t\x1a\x1a\n\tBytesList\x12\r\n\x05value\x18\x01 \x03(\x0c\x1a\x1e\n\tInt64List\x12\x11\n\x05value\x18\x01 \x03(\x03B\x02\x10\x01\x1a\x1e\n\tFloatList\x12\x11\n\x05value\x18\x01 \x03(\x02B\x02\x10\x01\x1a.\n\x07AnyList\x12#\n\x05value\x18\x01 \x03(\x0b2\x14.google.protobuf.AnyB\x06\n\x04kind"\xd1\x03\n\nTensorInfo\x12\x0e\n\x04name\x18\x01 \x01(\tH\x00\x126\n\ncoo_sparse\x18\x04 \x01(\x0b2 .tensorflow.TensorInfo.CooSparseH\x00\x12B\n\x10composite_tensor\x18\x05 \x01(\x0b2&.tensorflow.TensorInfo.CompositeTensorH\x00\x12#\n\x05dtype\x18\x02 \x01(\x0e2\x14.tensorflow.DataType\x122\n\x0ctensor_shape\x18\x03 \x01(\x0b2\x1c.tensorflow.TensorShapeProto\x1ae\n\tCooSparse\x12\x1a\n\x12values_tensor_name\x18\x01 \x01(\t\x12\x1b\n\x13indices_tensor_name\x18\x02 \x01(\t\x12\x1f\n\x17dense_shape_tensor_name\x18\x03 \x01(\t\x1ak\n\x0fCompositeTensor\x12,\n\ttype_spec\x18\x01 \x01(\x0b2\x19.tensorflow.TypeSpecProto\x12*\n\ncomponents\x18\x02 \x03(\x0b2\x16.tensorflow.TensorInfoB\n\n\x08encoding"\xa0\x02\n\x0cSignatureDef\x124\n\x06inputs\x18\x01 \x03(\x0b2$.tensorflow.SignatureDef.InputsEntry\x126\n\x07outputs\x18\x02 \x03(\x0b2%.tensorflow.SignatureDef.OutputsEntry\x12\x13\n\x0bmethod_name\x18\x03 \x01(\t\x1aE\n\x0bInputsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12%\n\x05value\x18\x02 \x01(\x0b2\x16.tensorflow.TensorInfo:\x028\x01\x1aF\n\x0cOutputsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12%\n\x05value\x18\x02 \x01(\x0b2\x16.tensorflow.TensorInfo:\x028\x01"M\n\x0cAssetFileDef\x12+\n\x0btensor_info\x18\x01 \x01(\x0b2\x16.tensorflow.TensorInfo\x12\x10\n\x08filename\x18\x02 \x01(\tB\x87\x01\n\x18org.tensorflow.frameworkB\x0fMetaGraphProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01b\x06proto3', dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR, tensorflow_dot_core_dot_framework_dot_graph__pb2.DESCRIPTOR, tensorflow_dot_core_dot_framework_dot_op__def__pb2.DESCRIPTOR, tensorflow_dot_core_dot_framework_dot_tensor__shape__pb2.DESCRIPTOR, tensorflow_dot_core_dot_framework_dot_types__pb2.DESCRIPTOR, tensorflow_dot_core_dot_protobuf_dot_saved__object__graph__pb2.DESCRIPTOR, tensorflow_dot_core_dot_protobuf_dot_saver__pb2.DESCRIPTOR, tensorflow_dot_core_dot_protobuf_dot_struct__pb2.DESCRIPTOR])
_METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY = _descriptor.Descriptor(name='FunctionAliasesEntry', full_name='tensorflow.MetaGraphDef.MetaInfoDef.FunctionAliasesEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.MetaGraphDef.MetaInfoDef.FunctionAliasesEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.MetaGraphDef.MetaInfoDef.FunctionAliasesEntry.value', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1099, serialized_end=1153)
_METAGRAPHDEF_METAINFODEF = _descriptor.Descriptor(name='MetaInfoDef', full_name='tensorflow.MetaGraphDef.MetaInfoDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='meta_graph_version', full_name='tensorflow.MetaGraphDef.MetaInfoDef.meta_graph_version', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='stripped_op_list', full_name='tensorflow.MetaGraphDef.MetaInfoDef.stripped_op_list', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='any_info', full_name='tensorflow.MetaGraphDef.MetaInfoDef.any_info', index=2, number=3, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='tags', full_name='tensorflow.MetaGraphDef.MetaInfoDef.tags', index=3, number=4, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='tensorflow_version', full_name='tensorflow.MetaGraphDef.MetaInfoDef.tensorflow_version', index=4, number=5, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='tensorflow_git_version', full_name='tensorflow.MetaGraphDef.MetaInfoDef.tensorflow_git_version', index=5, number=6, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='stripped_default_attrs', full_name='tensorflow.MetaGraphDef.MetaInfoDef.stripped_default_attrs', index=6, number=7, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='function_aliases', full_name='tensorflow.MetaGraphDef.MetaInfoDef.function_aliases', index=7, number=8, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=779, serialized_end=1153)
_METAGRAPHDEF_COLLECTIONDEFENTRY = _descriptor.Descriptor(name='CollectionDefEntry', full_name='tensorflow.MetaGraphDef.CollectionDefEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.MetaGraphDef.CollectionDefEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.MetaGraphDef.CollectionDefEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1155, serialized_end=1234)
_METAGRAPHDEF_SIGNATUREDEFENTRY = _descriptor.Descriptor(name='SignatureDefEntry', full_name='tensorflow.MetaGraphDef.SignatureDefEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.MetaGraphDef.SignatureDefEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.MetaGraphDef.SignatureDefEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1236, serialized_end=1313)
_METAGRAPHDEF = _descriptor.Descriptor(name='MetaGraphDef', full_name='tensorflow.MetaGraphDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='meta_info_def', full_name='tensorflow.MetaGraphDef.meta_info_def', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='graph_def', full_name='tensorflow.MetaGraphDef.graph_def', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='saver_def', full_name='tensorflow.MetaGraphDef.saver_def', index=2, number=3, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='collection_def', full_name='tensorflow.MetaGraphDef.collection_def', index=3, number=4, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='signature_def', full_name='tensorflow.MetaGraphDef.signature_def', index=4, number=5, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='asset_file_def', full_name='tensorflow.MetaGraphDef.asset_file_def', index=5, number=6, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='object_graph_def', full_name='tensorflow.MetaGraphDef.object_graph_def', index=6, number=7, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_METAGRAPHDEF_METAINFODEF, _METAGRAPHDEF_COLLECTIONDEFENTRY, _METAGRAPHDEF_SIGNATUREDEFENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=377, serialized_end=1313)
_COLLECTIONDEF_NODELIST = _descriptor.Descriptor(name='NodeList', full_name='tensorflow.CollectionDef.NodeList', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='value', full_name='tensorflow.CollectionDef.NodeList.value', index=0, number=1, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1622, serialized_end=1647)
_COLLECTIONDEF_BYTESLIST = _descriptor.Descriptor(name='BytesList', full_name='tensorflow.CollectionDef.BytesList', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='value', full_name='tensorflow.CollectionDef.BytesList.value', index=0, number=1, type=12, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1649, serialized_end=1675)
_COLLECTIONDEF_INT64LIST = _descriptor.Descriptor(name='Int64List', full_name='tensorflow.CollectionDef.Int64List', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='value', full_name='tensorflow.CollectionDef.Int64List.value', index=0, number=1, type=3, cpp_type=2, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=b'\x10\x01', file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1677, serialized_end=1707)
_COLLECTIONDEF_FLOATLIST = _descriptor.Descriptor(name='FloatList', full_name='tensorflow.CollectionDef.FloatList', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='value', full_name='tensorflow.CollectionDef.FloatList.value', index=0, number=1, type=2, cpp_type=6, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=b'\x10\x01', file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1709, serialized_end=1739)
_COLLECTIONDEF_ANYLIST = _descriptor.Descriptor(name='AnyList', full_name='tensorflow.CollectionDef.AnyList', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='value', full_name='tensorflow.CollectionDef.AnyList.value', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1741, serialized_end=1787)
_COLLECTIONDEF = _descriptor.Descriptor(name='CollectionDef', full_name='tensorflow.CollectionDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='node_list', full_name='tensorflow.CollectionDef.node_list', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='bytes_list', full_name='tensorflow.CollectionDef.bytes_list', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='int64_list', full_name='tensorflow.CollectionDef.int64_list', index=2, number=3, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='float_list', full_name='tensorflow.CollectionDef.float_list', index=3, number=4, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='any_list', full_name='tensorflow.CollectionDef.any_list', index=4, number=5, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_COLLECTIONDEF_NODELIST, _COLLECTIONDEF_BYTESLIST, _COLLECTIONDEF_INT64LIST, _COLLECTIONDEF_FLOATLIST, _COLLECTIONDEF_ANYLIST], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[_descriptor.OneofDescriptor(name='kind', full_name='tensorflow.CollectionDef.kind', index=0, containing_type=None, create_key=_descriptor._internal_create_key, fields=[])], serialized_start=1316, serialized_end=1795)
_TENSORINFO_COOSPARSE = _descriptor.Descriptor(name='CooSparse', full_name='tensorflow.TensorInfo.CooSparse', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='values_tensor_name', full_name='tensorflow.TensorInfo.CooSparse.values_tensor_name', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='indices_tensor_name', full_name='tensorflow.TensorInfo.CooSparse.indices_tensor_name', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='dense_shape_tensor_name', full_name='tensorflow.TensorInfo.CooSparse.dense_shape_tensor_name', index=2, number=3, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=2041, serialized_end=2142)
_TENSORINFO_COMPOSITETENSOR = _descriptor.Descriptor(name='CompositeTensor', full_name='tensorflow.TensorInfo.CompositeTensor', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='type_spec', full_name='tensorflow.TensorInfo.CompositeTensor.type_spec', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='components', full_name='tensorflow.TensorInfo.CompositeTensor.components', index=1, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=2144, serialized_end=2251)
_TENSORINFO = _descriptor.Descriptor(name='TensorInfo', full_name='tensorflow.TensorInfo', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='name', full_name='tensorflow.TensorInfo.name', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='coo_sparse', full_name='tensorflow.TensorInfo.coo_sparse', index=1, number=4, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='composite_tensor', full_name='tensorflow.TensorInfo.composite_tensor', index=2, number=5, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='dtype', full_name='tensorflow.TensorInfo.dtype', index=3, number=2, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='tensor_shape', full_name='tensorflow.TensorInfo.tensor_shape', index=4, number=3, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_TENSORINFO_COOSPARSE, _TENSORINFO_COMPOSITETENSOR], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[_descriptor.OneofDescriptor(name='encoding', full_name='tensorflow.TensorInfo.encoding', index=0, containing_type=None, create_key=_descriptor._internal_create_key, fields=[])], serialized_start=1798, serialized_end=2263)
_SIGNATUREDEF_INPUTSENTRY = _descriptor.Descriptor(name='InputsEntry', full_name='tensorflow.SignatureDef.InputsEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.SignatureDef.InputsEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.SignatureDef.InputsEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=2413, serialized_end=2482)
_SIGNATUREDEF_OUTPUTSENTRY = _descriptor.Descriptor(name='OutputsEntry', full_name='tensorflow.SignatureDef.OutputsEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.SignatureDef.OutputsEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.SignatureDef.OutputsEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=2484, serialized_end=2554)
_SIGNATUREDEF = _descriptor.Descriptor(name='SignatureDef', full_name='tensorflow.SignatureDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='inputs', full_name='tensorflow.SignatureDef.inputs', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='outputs', full_name='tensorflow.SignatureDef.outputs', index=1, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='method_name', full_name='tensorflow.SignatureDef.method_name', index=2, number=3, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_SIGNATUREDEF_INPUTSENTRY, _SIGNATUREDEF_OUTPUTSENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=2266, serialized_end=2554)
_ASSETFILEDEF = _descriptor.Descriptor(name='AssetFileDef', full_name='tensorflow.AssetFileDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='tensor_info', full_name='tensorflow.AssetFileDef.tensor_info', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='filename', full_name='tensorflow.AssetFileDef.filename', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=2556, serialized_end=2633)
_METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY.containing_type = _METAGRAPHDEF_METAINFODEF
_METAGRAPHDEF_METAINFODEF.fields_by_name['stripped_op_list'].message_type = tensorflow_dot_core_dot_framework_dot_op__def__pb2._OPLIST
_METAGRAPHDEF_METAINFODEF.fields_by_name['any_info'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_METAGRAPHDEF_METAINFODEF.fields_by_name['function_aliases'].message_type = _METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY
_METAGRAPHDEF_METAINFODEF.containing_type = _METAGRAPHDEF
_METAGRAPHDEF_COLLECTIONDEFENTRY.fields_by_name['value'].message_type = _COLLECTIONDEF
_METAGRAPHDEF_COLLECTIONDEFENTRY.containing_type = _METAGRAPHDEF
_METAGRAPHDEF_SIGNATUREDEFENTRY.fields_by_name['value'].message_type = _SIGNATUREDEF
_METAGRAPHDEF_SIGNATUREDEFENTRY.containing_type = _METAGRAPHDEF
_METAGRAPHDEF.fields_by_name['meta_info_def'].message_type = _METAGRAPHDEF_METAINFODEF
_METAGRAPHDEF.fields_by_name['graph_def'].message_type = tensorflow_dot_core_dot_framework_dot_graph__pb2._GRAPHDEF
_METAGRAPHDEF.fields_by_name['saver_def'].message_type = tensorflow_dot_core_dot_protobuf_dot_saver__pb2._SAVERDEF
_METAGRAPHDEF.fields_by_name['collection_def'].message_type = _METAGRAPHDEF_COLLECTIONDEFENTRY
_METAGRAPHDEF.fields_by_name['signature_def'].message_type = _METAGRAPHDEF_SIGNATUREDEFENTRY
_METAGRAPHDEF.fields_by_name['asset_file_def'].message_type = _ASSETFILEDEF
_METAGRAPHDEF.fields_by_name['object_graph_def'].message_type = tensorflow_dot_core_dot_protobuf_dot_saved__object__graph__pb2._SAVEDOBJECTGRAPH
_COLLECTIONDEF_NODELIST.containing_type = _COLLECTIONDEF
_COLLECTIONDEF_BYTESLIST.containing_type = _COLLECTIONDEF
_COLLECTIONDEF_INT64LIST.containing_type = _COLLECTIONDEF
_COLLECTIONDEF_FLOATLIST.containing_type = _COLLECTIONDEF
_COLLECTIONDEF_ANYLIST.fields_by_name['value'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_COLLECTIONDEF_ANYLIST.containing_type = _COLLECTIONDEF
_COLLECTIONDEF.fields_by_name['node_list'].message_type = _COLLECTIONDEF_NODELIST
_COLLECTIONDEF.fields_by_name['bytes_list'].message_type = _COLLECTIONDEF_BYTESLIST
_COLLECTIONDEF.fields_by_name['int64_list'].message_type = _COLLECTIONDEF_INT64LIST
_COLLECTIONDEF.fields_by_name['float_list'].message_type = _COLLECTIONDEF_FLOATLIST
_COLLECTIONDEF.fields_by_name['any_list'].message_type = _COLLECTIONDEF_ANYLIST
_COLLECTIONDEF.oneofs_by_name['kind'].fields.append(_COLLECTIONDEF.fields_by_name['node_list'])
_COLLECTIONDEF.fields_by_name['node_list'].containing_oneof = _COLLECTIONDEF.oneofs_by_name['kind']
_COLLECTIONDEF.oneofs_by_name['kind'].fields.append(_COLLECTIONDEF.fields_by_name['bytes_list'])
_COLLECTIONDEF.fields_by_name['bytes_list'].containing_oneof = _COLLECTIONDEF.oneofs_by_name['kind']
_COLLECTIONDEF.oneofs_by_name['kind'].fields.append(_COLLECTIONDEF.fields_by_name['int64_list'])
_COLLECTIONDEF.fields_by_name['int64_list'].containing_oneof = _COLLECTIONDEF.oneofs_by_name['kind']
_COLLECTIONDEF.oneofs_by_name['kind'].fields.append(_COLLECTIONDEF.fields_by_name['float_list'])
_COLLECTIONDEF.fields_by_name['float_list'].containing_oneof = _COLLECTIONDEF.oneofs_by_name['kind']
_COLLECTIONDEF.oneofs_by_name['kind'].fields.append(_COLLECTIONDEF.fields_by_name['any_list'])
_COLLECTIONDEF.fields_by_name['any_list'].containing_oneof = _COLLECTIONDEF.oneofs_by_name['kind']
_TENSORINFO_COOSPARSE.containing_type = _TENSORINFO
_TENSORINFO_COMPOSITETENSOR.fields_by_name['type_spec'].message_type = tensorflow_dot_core_dot_protobuf_dot_struct__pb2._TYPESPECPROTO
_TENSORINFO_COMPOSITETENSOR.fields_by_name['components'].message_type = _TENSORINFO
_TENSORINFO_COMPOSITETENSOR.containing_type = _TENSORINFO
_TENSORINFO.fields_by_name['coo_sparse'].message_type = _TENSORINFO_COOSPARSE
_TENSORINFO.fields_by_name['composite_tensor'].message_type = _TENSORINFO_COMPOSITETENSOR
_TENSORINFO.fields_by_name['dtype'].enum_type = tensorflow_dot_core_dot_framework_dot_types__pb2._DATATYPE
_TENSORINFO.fields_by_name['tensor_shape'].message_type = tensorflow_dot_core_dot_framework_dot_tensor__shape__pb2._TENSORSHAPEPROTO
_TENSORINFO.oneofs_by_name['encoding'].fields.append(_TENSORINFO.fields_by_name['name'])
_TENSORINFO.fields_by_name['name'].containing_oneof = _TENSORINFO.oneofs_by_name['encoding']
_TENSORINFO.oneofs_by_name['encoding'].fields.append(_TENSORINFO.fields_by_name['coo_sparse'])
_TENSORINFO.fields_by_name['coo_sparse'].containing_oneof = _TENSORINFO.oneofs_by_name['encoding']
_TENSORINFO.oneofs_by_name['encoding'].fields.append(_TENSORINFO.fields_by_name['composite_tensor'])
_TENSORINFO.fields_by_name['composite_tensor'].containing_oneof = _TENSORINFO.oneofs_by_name['encoding']
_SIGNATUREDEF_INPUTSENTRY.fields_by_name['value'].message_type = _TENSORINFO
_SIGNATUREDEF_INPUTSENTRY.containing_type = _SIGNATUREDEF
_SIGNATUREDEF_OUTPUTSENTRY.fields_by_name['value'].message_type = _TENSORINFO
_SIGNATUREDEF_OUTPUTSENTRY.containing_type = _SIGNATUREDEF
_SIGNATUREDEF.fields_by_name['inputs'].message_type = _SIGNATUREDEF_INPUTSENTRY
_SIGNATUREDEF.fields_by_name['outputs'].message_type = _SIGNATUREDEF_OUTPUTSENTRY
_ASSETFILEDEF.fields_by_name['tensor_info'].message_type = _TENSORINFO
DESCRIPTOR.message_types_by_name['MetaGraphDef'] = _METAGRAPHDEF
DESCRIPTOR.message_types_by_name['CollectionDef'] = _COLLECTIONDEF
DESCRIPTOR.message_types_by_name['TensorInfo'] = _TENSORINFO
DESCRIPTOR.message_types_by_name['SignatureDef'] = _SIGNATUREDEF
DESCRIPTOR.message_types_by_name['AssetFileDef'] = _ASSETFILEDEF
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
MetaGraphDef = _reflection.GeneratedProtocolMessageType('MetaGraphDef', (_message.Message,), {'MetaInfoDef': _reflection.GeneratedProtocolMessageType('MetaInfoDef', (_message.Message,), {'FunctionAliasesEntry': _reflection.GeneratedProtocolMessageType('FunctionAliasesEntry', (_message.Message,), {'DESCRIPTOR': _METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'DESCRIPTOR': _METAGRAPHDEF_METAINFODEF, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'CollectionDefEntry': _reflection.GeneratedProtocolMessageType('CollectionDefEntry', (_message.Message,), {'DESCRIPTOR': _METAGRAPHDEF_COLLECTIONDEFENTRY, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'SignatureDefEntry': _reflection.GeneratedProtocolMessageType('SignatureDefEntry', (_message.Message,), {'DESCRIPTOR': _METAGRAPHDEF_SIGNATUREDEFENTRY, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'DESCRIPTOR': _METAGRAPHDEF, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'})
_sym_db.RegisterMessage(MetaGraphDef)
_sym_db.RegisterMessage(MetaGraphDef.MetaInfoDef)
_sym_db.RegisterMessage(MetaGraphDef.MetaInfoDef.FunctionAliasesEntry)
_sym_db.RegisterMessage(MetaGraphDef.CollectionDefEntry)
_sym_db.RegisterMessage(MetaGraphDef.SignatureDefEntry)
CollectionDef = _reflection.GeneratedProtocolMessageType('CollectionDef', (_message.Message,), {'NodeList': _reflection.GeneratedProtocolMessageType('NodeList', (_message.Message,), {'DESCRIPTOR': _COLLECTIONDEF_NODELIST, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'BytesList': _reflection.GeneratedProtocolMessageType('BytesList', (_message.Message,), {'DESCRIPTOR': _COLLECTIONDEF_BYTESLIST, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'Int64List': _reflection.GeneratedProtocolMessageType('Int64List', (_message.Message,), {'DESCRIPTOR': _COLLECTIONDEF_INT64LIST, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'FloatList': _reflection.GeneratedProtocolMessageType('FloatList', (_message.Message,), {'DESCRIPTOR': _COLLECTIONDEF_FLOATLIST, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'AnyList': _reflection.GeneratedProtocolMessageType('AnyList', (_message.Message,), {'DESCRIPTOR': _COLLECTIONDEF_ANYLIST, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'DESCRIPTOR': _COLLECTIONDEF, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'})
_sym_db.RegisterMessage(CollectionDef)
_sym_db.RegisterMessage(CollectionDef.NodeList)
_sym_db.RegisterMessage(CollectionDef.BytesList)
_sym_db.RegisterMessage(CollectionDef.Int64List)
_sym_db.RegisterMessage(CollectionDef.FloatList)
_sym_db.RegisterMessage(CollectionDef.AnyList)
TensorInfo = _reflection.GeneratedProtocolMessageType('TensorInfo', (_message.Message,), {'CooSparse': _reflection.GeneratedProtocolMessageType('CooSparse', (_message.Message,), {'DESCRIPTOR': _TENSORINFO_COOSPARSE, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'CompositeTensor': _reflection.GeneratedProtocolMessageType('CompositeTensor', (_message.Message,), {'DESCRIPTOR': _TENSORINFO_COMPOSITETENSOR, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'DESCRIPTOR': _TENSORINFO, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'})
_sym_db.RegisterMessage(TensorInfo)
_sym_db.RegisterMessage(TensorInfo.CooSparse)
_sym_db.RegisterMessage(TensorInfo.CompositeTensor)
SignatureDef = _reflection.GeneratedProtocolMessageType('SignatureDef', (_message.Message,), {'InputsEntry': _reflection.GeneratedProtocolMessageType('InputsEntry', (_message.Message,), {'DESCRIPTOR': _SIGNATUREDEF_INPUTSENTRY, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'OutputsEntry': _reflection.GeneratedProtocolMessageType('OutputsEntry', (_message.Message,), {'DESCRIPTOR': _SIGNATUREDEF_OUTPUTSENTRY, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'}), 'DESCRIPTOR': _SIGNATUREDEF, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'})
_sym_db.RegisterMessage(SignatureDef)
_sym_db.RegisterMessage(SignatureDef.InputsEntry)
_sym_db.RegisterMessage(SignatureDef.OutputsEntry)
AssetFileDef = _reflection.GeneratedProtocolMessageType('AssetFileDef', (_message.Message,), {'DESCRIPTOR': _ASSETFILEDEF, '__module__': 'tensorflow.core.protobuf.meta_graph_pb2'})
_sym_db.RegisterMessage(AssetFileDef)
DESCRIPTOR._options = None
_METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY._options = None
_METAGRAPHDEF_COLLECTIONDEFENTRY._options = None
_METAGRAPHDEF_SIGNATUREDEFENTRY._options = None
_COLLECTIONDEF_INT64LIST.fields_by_name['value']._options = None
_COLLECTIONDEF_FLOATLIST.fields_by_name['value']._options = None
_SIGNATUREDEF_INPUTSENTRY._options = None
_SIGNATUREDEF_OUTPUTSENTRY._options = None




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
from ....tensorflow.core.example import feature_pb2 as tensorflow_dot_core_dot_example_dot_feature__pb2
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/example/example.proto', package='tensorflow', syntax='proto3', serialized_options=b'\n\x16org.tensorflow.exampleB\rExampleProtosP\x01ZSgithub.com/tensorflow/tensorflow/tensorflow/go/core/example/example_protos_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n%tensorflow/core/example/example.proto\x12\ntensorflow\x1a%tensorflow/core/example/feature.proto"1\n\x07Example\x12&\n\x08features\x18\x01 \x01(\x0b2\x14.tensorflow.Features"i\n\x0fSequenceExample\x12%\n\x07context\x18\x01 \x01(\x0b2\x14.tensorflow.Features\x12/\n\rfeature_lists\x18\x02 \x01(\x0b2\x18.tensorflow.FeatureListsB\x81\x01\n\x16org.tensorflow.exampleB\rExampleProtosP\x01ZSgithub.com/tensorflow/tensorflow/tensorflow/go/core/example/example_protos_go_proto\xf8\x01\x01b\x06proto3', dependencies=[tensorflow_dot_core_dot_example_dot_feature__pb2.DESCRIPTOR])
_EXAMPLE = _descriptor.Descriptor(name='Example', full_name='tensorflow.Example', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='features', full_name='tensorflow.Example.features', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=92, serialized_end=141)
_SEQUENCEEXAMPLE = _descriptor.Descriptor(name='SequenceExample', full_name='tensorflow.SequenceExample', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='context', full_name='tensorflow.SequenceExample.context', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='feature_lists', full_name='tensorflow.SequenceExample.feature_lists', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=143, serialized_end=248)
_EXAMPLE.fields_by_name['features'].message_type = tensorflow_dot_core_dot_example_dot_feature__pb2._FEATURES
_SEQUENCEEXAMPLE.fields_by_name['context'].message_type = tensorflow_dot_core_dot_example_dot_feature__pb2._FEATURES
_SEQUENCEEXAMPLE.fields_by_name['feature_lists'].message_type = tensorflow_dot_core_dot_example_dot_feature__pb2._FEATURELISTS
DESCRIPTOR.message_types_by_name['Example'] = _EXAMPLE
DESCRIPTOR.message_types_by_name['SequenceExample'] = _SEQUENCEEXAMPLE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
Example = _reflection.GeneratedProtocolMessageType('Example', (_message.Message,), {'DESCRIPTOR': _EXAMPLE, '__module__': 'tensorflow.core.example.example_pb2'})
_sym_db.RegisterMessage(Example)
SequenceExample = _reflection.GeneratedProtocolMessageType('SequenceExample', (_message.Message,), {'DESCRIPTOR': _SEQUENCEEXAMPLE, '__module__': 'tensorflow.core.example.example_pb2'})
_sym_db.RegisterMessage(SequenceExample)
DESCRIPTOR._options = None



from .pb.tensorflow.core.framework import types_pb2
from . import tensor_shape
from . import errors
from . import numpy_util
from . import core

from typing import Tuple, Optional

import numpy as np


class TensorBuffer:
  def __init__(self, data=None, array_offset=0, order="C"):
    self._data = data
    self._array_offset = array_offset
    self._order = order
    # verify our data is OK; this runs various assertions.
    self.data()

  def data(self) -> Optional[memoryview]:
    data = self._data
    if data is None:
      return data
    # try to get a JAX Array as a numpy array
    if hasattr(data, '_value'):
      data = data._value
    if isinstance(data, np.ndarray):
      data = data.data
    if isinstance(data, memoryview):
      assert data.contiguous, "memory must be contiguous"
      assert data.c_contiguous, "memory must be C contiguous"
    return data

  def array_offset(self):
    return self._array_offset

  def order(self):
    return self._order

  def total_bytes(self) -> int:
    if (data := self.data()) is None:
      return 0
    return data.nbytes


class Tensor:
  def __init__(self,
               type: types_pb2.DataType = types_pb2.DT_FLOAT,
               shape: tensor_shape.TensorShape = None,
               buf: TensorBuffer = None):
    if shape is None:
      shape = tensor_shape.TensorShape()
    shape.set_data_type(type)
    self._shape = shape
    self._buf = buf

  @classmethod
  def from_array(cls, array):
    type = errors.raise_if_error(numpy_util.numpy_dtype_to_data_type(array.dtype))
    shape = tensor_shape.TensorShape.from_dims(array.shape)
    buf = TensorBuffer(array)
    return cls(type=type, shape=shape, buf=buf)

  def __repr__(self):
    return f"Tensor(is_initialized={self.is_initialized()}, shape={self.shape()!r})"

  def shape(self) -> tensor_shape.TensorShape:
    return self._shape

  def dtype(self) -> types_pb2.DataType:
    return self.shape().data_type()

  def dims(self) -> int:
    return self.shape().dims()

  def dim_size(self, d: int) -> int:
    return self.shape().dim_size(d)

  def num_elements(self) -> int:
    return self.shape().num_elements()

  def is_initialized(self) -> bool:
    """If necessary, has this Tensor been initialized?

    Zero-element Tensors are always considered initialized, even if they
    have never been assigned to and do not have any memory allocated."""
    return (self._buf is not None and self._buf.data() is not None) or self.num_elements() == 0

  def set_data(self, buf: TensorBuffer):
    self._buf = buf

  def tensor_data(self) -> core.StringPiece:
    if self._buf is None:
      # Don't die for empty tensors
      return core.StringPiece()
    # return StringPiece(static_cast<char*>(buf_->data()), TotalBytes());
    return core.StringPiece(self._buf.data(), self.total_bytes())

  # size_t Tensor::TotalBytes() const {
  #   if (shape_.num_elements() == 0) return 0;
  #   CHECK(buf_) << "null buf_ with non-zero shape size " << shape_.num_elements();
  #   CASES(dtype(), return Helper<T>::TotalBytes(buf_, shape_.num_elements()));
  #   return 0;  // Makes compiler happy.
  # }
  def total_bytes(self) -> int:
    numel = self._shape.num_elements()
    if numel == 0:
      return 0
    assert self._buf is not None, "null _buf with non-zero shape size"
    # elemsize = np.dtype(self.shape().dtype()).itemsize
    # return elemsize * numel
    return self._buf.total_bytes()

  def to_py(self) -> Tuple[errors.Status, Optional[np.ndarray]]:
    if not self.is_initialized():
      return errors.FailedPrecondition("Not initialized"), None
    shape = self.shape()
    dtype = shape.dtype()
    if dtype is None:
      return errors.Unimplemented("Data type ", shape.data_type(), " not supported"), None
    if self.num_elements() != 0:
      arr = np.ndarray.__new__(np.ndarray,
                               shape=shape.to_list(),
                               dtype=dtype,
                               buffer=self._buf.data(),
                               offset=self._buf.array_offset(),
                               order=self._buf.order())
      # a = np.frombuffer(self._buf.data(), dtype, shape.num_elements())
      # a = a.reshape(shape.to_list())
    else:
      arr = np.zeros(shape.to_list(), dtype)
    return errors.Status.OK(), arr




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/compiler/xla/pjrt/distributed/protocol.proto', package='xla', syntax='proto3', serialized_options=None, create_key=_descriptor._internal_create_key, serialized_pb=b'\n7tensorflow/compiler/xla/pjrt/distributed/protocol.proto\x12\x03xla"c\n\x0bDeviceProto\x12\x1c\n\x14local_device_ordinal\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0e\n\x06vendor\x18\x03 \x01(\t\x12\x18\n\x10global_device_id\x18\x04 \x01(\x05"H\n\x12LocalTopologyProto\x12\x0f\n\x07node_id\x18\x01 \x01(\x05\x12!\n\x07devices\x18\x02 \x03(\x0b2\x10.xla.DeviceProto"=\n\x13GlobalTopologyProto\x12&\n\x05nodes\x18\x01 \x03(\x0b2\x17.xla.LocalTopologyProto"l\n\x0eConnectRequest\x12\x18\n\x10protocol_version\x18\x01 \x01(\x05\x12\x1c\n\x14timeout_milliseconds\x18\x02 \x01(\x05\x12\x0f\n\x07node_id\x18\x03 \x01(\x05\x12\x11\n\tclient_id\x18\x04 \x01(\x04"%\n\x0fConnectResponse\x12\x12\n\nsession_id\x18\x01 \x01(\x04"^\n\x17EnumerateDevicesRequest\x12\x12\n\nsession_id\x18\x01 \x01(\x04\x12/\n\x0elocal_topology\x18\x03 \x01(\x0b2\x17.xla.LocalTopologyProto"M\n\x18EnumerateDevicesResponse\x121\n\x0fglobal_topology\x18\x01 \x01(\x0b2\x18.xla.GlobalTopologyProto"S\n\x12KeyValueGetRequest\x12\x12\n\nsession_id\x18\x01 \x01(\x04\x12\x0b\n\x03key\x18\x02 \x01(\x0c\x12\x1c\n\x14timeout_milliseconds\x18\x03 \x01(\x05"3\n\x13KeyValueGetResponse\x12\r\n\x05found\x18\x01 \x01(\x08\x12\r\n\x05value\x18\x02 \x01(\x0c"D\n\x12KeyValueSetRequest\x12\x12\n\nsession_id\x18\x01 \x01(\x04\x12\x0b\n\x03key\x18\x02 \x01(\x0c\x12\r\n\x05value\x18\x03 \x01(\x0c"\x15\n\x13KeyValueSetResponse"7\n\x10HeartbeatRequest\x12\x12\n\nsession_id\x18\x01 \x01(\x04\x12\x0f\n\x07node_id\x18\x02 \x01(\x05"\x13\n\x11HeartbeatResponse"6\n\x0fShutdownRequest\x12\x12\n\nsession_id\x18\x01 \x01(\x04\x12\x0f\n\x07node_id\x18\x02 \x01(\x05"\x12\n\x10ShutdownResponse2\xa7\x03\n\x19DistributedRuntimeService\x126\n\x07Connect\x12\x13.xla.ConnectRequest\x1a\x14.xla.ConnectResponse"\x00\x12Q\n\x10EnumerateDevices\x12\x1c.xla.EnumerateDevicesRequest\x1a\x1d.xla.EnumerateDevicesResponse"\x00\x12<\n\tHeartbeat\x12\x15.xla.HeartbeatRequest\x1a\x16.xla.HeartbeatResponse"\x00\x129\n\x08Shutdown\x12\x14.xla.ShutdownRequest\x1a\x15.xla.ShutdownResponse"\x00\x12B\n\x0bKeyValueGet\x12\x17.xla.KeyValueGetRequest\x1a\x18.xla.KeyValueGetResponse"\x00\x12B\n\x0bKeyValueSet\x12\x17.xla.KeyValueSetRequest\x1a\x18.xla.KeyValueSetResponse"\x00b\x06proto3')
_DEVICEPROTO = _descriptor.Descriptor(name='DeviceProto', full_name='xla.DeviceProto', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='local_device_ordinal', full_name='xla.DeviceProto.local_device_ordinal', index=0, number=1, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='name', full_name='xla.DeviceProto.name', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='vendor', full_name='xla.DeviceProto.vendor', index=2, number=3, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='global_device_id', full_name='xla.DeviceProto.global_device_id', index=3, number=4, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=64, serialized_end=163)
_LOCALTOPOLOGYPROTO = _descriptor.Descriptor(name='LocalTopologyProto', full_name='xla.LocalTopologyProto', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='node_id', full_name='xla.LocalTopologyProto.node_id', index=0, number=1, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='devices', full_name='xla.LocalTopologyProto.devices', index=1, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=165, serialized_end=237)
_GLOBALTOPOLOGYPROTO = _descriptor.Descriptor(name='GlobalTopologyProto', full_name='xla.GlobalTopologyProto', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='nodes', full_name='xla.GlobalTopologyProto.nodes', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=239, serialized_end=300)
_CONNECTREQUEST = _descriptor.Descriptor(name='ConnectRequest', full_name='xla.ConnectRequest', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='protocol_version', full_name='xla.ConnectRequest.protocol_version', index=0, number=1, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='timeout_milliseconds', full_name='xla.ConnectRequest.timeout_milliseconds', index=1, number=2, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='node_id', full_name='xla.ConnectRequest.node_id', index=2, number=3, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='client_id', full_name='xla.ConnectRequest.client_id', index=3, number=4, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=302, serialized_end=410)
_CONNECTRESPONSE = _descriptor.Descriptor(name='ConnectResponse', full_name='xla.ConnectResponse', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='session_id', full_name='xla.ConnectResponse.session_id', index=0, number=1, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=412, serialized_end=449)
_ENUMERATEDEVICESREQUEST = _descriptor.Descriptor(name='EnumerateDevicesRequest', full_name='xla.EnumerateDevicesRequest', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='session_id', full_name='xla.EnumerateDevicesRequest.session_id', index=0, number=1, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='local_topology', full_name='xla.EnumerateDevicesRequest.local_topology', index=1, number=3, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=451, serialized_end=545)
_ENUMERATEDEVICESRESPONSE = _descriptor.Descriptor(name='EnumerateDevicesResponse', full_name='xla.EnumerateDevicesResponse', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='global_topology', full_name='xla.EnumerateDevicesResponse.global_topology', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=547, serialized_end=624)
_KEYVALUEGETREQUEST = _descriptor.Descriptor(name='KeyValueGetRequest', full_name='xla.KeyValueGetRequest', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='session_id', full_name='xla.KeyValueGetRequest.session_id', index=0, number=1, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='key', full_name='xla.KeyValueGetRequest.key', index=1, number=2, type=12, cpp_type=9, label=1, has_default_value=False, default_value=b'', message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='timeout_milliseconds', full_name='xla.KeyValueGetRequest.timeout_milliseconds', index=2, number=3, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=626, serialized_end=709)
_KEYVALUEGETRESPONSE = _descriptor.Descriptor(name='KeyValueGetResponse', full_name='xla.KeyValueGetResponse', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='found', full_name='xla.KeyValueGetResponse.found', index=0, number=1, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='xla.KeyValueGetResponse.value', index=1, number=2, type=12, cpp_type=9, label=1, has_default_value=False, default_value=b'', message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=711, serialized_end=762)
_KEYVALUESETREQUEST = _descriptor.Descriptor(name='KeyValueSetRequest', full_name='xla.KeyValueSetRequest', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='session_id', full_name='xla.KeyValueSetRequest.session_id', index=0, number=1, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='key', full_name='xla.KeyValueSetRequest.key', index=1, number=2, type=12, cpp_type=9, label=1, has_default_value=False, default_value=b'', message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='xla.KeyValueSetRequest.value', index=2, number=3, type=12, cpp_type=9, label=1, has_default_value=False, default_value=b'', message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=764, serialized_end=832)
_KEYVALUESETRESPONSE = _descriptor.Descriptor(name='KeyValueSetResponse', full_name='xla.KeyValueSetResponse', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=834, serialized_end=855)
_HEARTBEATREQUEST = _descriptor.Descriptor(name='HeartbeatRequest', full_name='xla.HeartbeatRequest', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='session_id', full_name='xla.HeartbeatRequest.session_id', index=0, number=1, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='node_id', full_name='xla.HeartbeatRequest.node_id', index=1, number=2, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=857, serialized_end=912)
_HEARTBEATRESPONSE = _descriptor.Descriptor(name='HeartbeatResponse', full_name='xla.HeartbeatResponse', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=914, serialized_end=933)
_SHUTDOWNREQUEST = _descriptor.Descriptor(name='ShutdownRequest', full_name='xla.ShutdownRequest', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='session_id', full_name='xla.ShutdownRequest.session_id', index=0, number=1, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='node_id', full_name='xla.ShutdownRequest.node_id', index=1, number=2, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=935, serialized_end=989)
_SHUTDOWNRESPONSE = _descriptor.Descriptor(name='ShutdownResponse', full_name='xla.ShutdownResponse', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=991, serialized_end=1009)
_LOCALTOPOLOGYPROTO.fields_by_name['devices'].message_type = _DEVICEPROTO
_GLOBALTOPOLOGYPROTO.fields_by_name['nodes'].message_type = _LOCALTOPOLOGYPROTO
_ENUMERATEDEVICESREQUEST.fields_by_name['local_topology'].message_type = _LOCALTOPOLOGYPROTO
_ENUMERATEDEVICESRESPONSE.fields_by_name['global_topology'].message_type = _GLOBALTOPOLOGYPROTO
DESCRIPTOR.message_types_by_name['DeviceProto'] = _DEVICEPROTO
DESCRIPTOR.message_types_by_name['LocalTopologyProto'] = _LOCALTOPOLOGYPROTO
DESCRIPTOR.message_types_by_name['GlobalTopologyProto'] = _GLOBALTOPOLOGYPROTO
DESCRIPTOR.message_types_by_name['ConnectRequest'] = _CONNECTREQUEST
DESCRIPTOR.message_types_by_name['ConnectResponse'] = _CONNECTRESPONSE
DESCRIPTOR.message_types_by_name['EnumerateDevicesRequest'] = _ENUMERATEDEVICESREQUEST
DESCRIPTOR.message_types_by_name['EnumerateDevicesResponse'] = _ENUMERATEDEVICESRESPONSE
DESCRIPTOR.message_types_by_name['KeyValueGetRequest'] = _KEYVALUEGETREQUEST
DESCRIPTOR.message_types_by_name['KeyValueGetResponse'] = _KEYVALUEGETRESPONSE
DESCRIPTOR.message_types_by_name['KeyValueSetRequest'] = _KEYVALUESETREQUEST
DESCRIPTOR.message_types_by_name['KeyValueSetResponse'] = _KEYVALUESETRESPONSE
DESCRIPTOR.message_types_by_name['HeartbeatRequest'] = _HEARTBEATREQUEST
DESCRIPTOR.message_types_by_name['HeartbeatResponse'] = _HEARTBEATRESPONSE
DESCRIPTOR.message_types_by_name['ShutdownRequest'] = _SHUTDOWNREQUEST
DESCRIPTOR.message_types_by_name['ShutdownResponse'] = _SHUTDOWNRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
DeviceProto = _reflection.GeneratedProtocolMessageType('DeviceProto', (_message.Message,), {'DESCRIPTOR': _DEVICEPROTO, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(DeviceProto)
LocalTopologyProto = _reflection.GeneratedProtocolMessageType('LocalTopologyProto', (_message.Message,), {'DESCRIPTOR': _LOCALTOPOLOGYPROTO, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(LocalTopologyProto)
GlobalTopologyProto = _reflection.GeneratedProtocolMessageType('GlobalTopologyProto', (_message.Message,), {'DESCRIPTOR': _GLOBALTOPOLOGYPROTO, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(GlobalTopologyProto)
ConnectRequest = _reflection.GeneratedProtocolMessageType('ConnectRequest', (_message.Message,), {'DESCRIPTOR': _CONNECTREQUEST, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(ConnectRequest)
ConnectResponse = _reflection.GeneratedProtocolMessageType('ConnectResponse', (_message.Message,), {'DESCRIPTOR': _CONNECTRESPONSE, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(ConnectResponse)
EnumerateDevicesRequest = _reflection.GeneratedProtocolMessageType('EnumerateDevicesRequest', (_message.Message,), {'DESCRIPTOR': _ENUMERATEDEVICESREQUEST, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(EnumerateDevicesRequest)
EnumerateDevicesResponse = _reflection.GeneratedProtocolMessageType('EnumerateDevicesResponse', (_message.Message,), {'DESCRIPTOR': _ENUMERATEDEVICESRESPONSE, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(EnumerateDevicesResponse)
KeyValueGetRequest = _reflection.GeneratedProtocolMessageType('KeyValueGetRequest', (_message.Message,), {'DESCRIPTOR': _KEYVALUEGETREQUEST, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(KeyValueGetRequest)
KeyValueGetResponse = _reflection.GeneratedProtocolMessageType('KeyValueGetResponse', (_message.Message,), {'DESCRIPTOR': _KEYVALUEGETRESPONSE, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(KeyValueGetResponse)
KeyValueSetRequest = _reflection.GeneratedProtocolMessageType('KeyValueSetRequest', (_message.Message,), {'DESCRIPTOR': _KEYVALUESETREQUEST, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(KeyValueSetRequest)
KeyValueSetResponse = _reflection.GeneratedProtocolMessageType('KeyValueSetResponse', (_message.Message,), {'DESCRIPTOR': _KEYVALUESETRESPONSE, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(KeyValueSetResponse)
HeartbeatRequest = _reflection.GeneratedProtocolMessageType('HeartbeatRequest', (_message.Message,), {'DESCRIPTOR': _HEARTBEATREQUEST, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(HeartbeatRequest)
HeartbeatResponse = _reflection.GeneratedProtocolMessageType('HeartbeatResponse', (_message.Message,), {'DESCRIPTOR': _HEARTBEATRESPONSE, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(HeartbeatResponse)
ShutdownRequest = _reflection.GeneratedProtocolMessageType('ShutdownRequest', (_message.Message,), {'DESCRIPTOR': _SHUTDOWNREQUEST, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(ShutdownRequest)
ShutdownResponse = _reflection.GeneratedProtocolMessageType('ShutdownResponse', (_message.Message,), {'DESCRIPTOR': _SHUTDOWNRESPONSE, '__module__': 'tensorflow.compiler.xla.pjrt.distributed.protocol_pb2'})
_sym_db.RegisterMessage(ShutdownResponse)
_DISTRIBUTEDRUNTIMESERVICE = _descriptor.ServiceDescriptor(name='DistributedRuntimeService', full_name='xla.DistributedRuntimeService', file=DESCRIPTOR, index=0, serialized_options=None, create_key=_descriptor._internal_create_key, serialized_start=1012, serialized_end=1435, methods=[_descriptor.MethodDescriptor(name='Connect', full_name='xla.DistributedRuntimeService.Connect', index=0, containing_service=None, input_type=_CONNECTREQUEST, output_type=_CONNECTRESPONSE, serialized_options=None, create_key=_descriptor._internal_create_key), _descriptor.MethodDescriptor(name='EnumerateDevices', full_name='xla.DistributedRuntimeService.EnumerateDevices', index=1, containing_service=None, input_type=_ENUMERATEDEVICESREQUEST, output_type=_ENUMERATEDEVICESRESPONSE, serialized_options=None, create_key=_descriptor._internal_create_key), _descriptor.MethodDescriptor(name='Heartbeat', full_name='xla.DistributedRuntimeService.Heartbeat', index=2, containing_service=None, input_type=_HEARTBEATREQUEST, output_type=_HEARTBEATRESPONSE, serialized_options=None, create_key=_descriptor._internal_create_key), _descriptor.MethodDescriptor(name='Shutdown', full_name='xla.DistributedRuntimeService.Shutdown', index=3, containing_service=None, input_type=_SHUTDOWNREQUEST, output_type=_SHUTDOWNRESPONSE, serialized_options=None, create_key=_descriptor._internal_create_key), _descriptor.MethodDescriptor(name='KeyValueGet', full_name='xla.DistributedRuntimeService.KeyValueGet', index=4, containing_service=None, input_type=_KEYVALUEGETREQUEST, output_type=_KEYVALUEGETRESPONSE, serialized_options=None, create_key=_descriptor._internal_create_key), _descriptor.MethodDescriptor(name='KeyValueSet', full_name='xla.DistributedRuntimeService.KeyValueSet', index=5, containing_service=None, input_type=_KEYVALUESETREQUEST, output_type=_KEYVALUESETRESPONSE, serialized_options=None, create_key=_descriptor._internal_create_key)])
_sym_db.RegisterServiceDescriptor(_DISTRIBUTEDRUNTIMESERVICE)
DESCRIPTOR.services_by_name['DistributedRuntimeService'] = _DISTRIBUTEDRUNTIMESERVICE




'Generated protocol buffer code.'
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/protobuf/error_codes.proto', package='tensorflow.error', syntax='proto3', serialized_options=b'\n\x18org.tensorflow.frameworkB\x10ErrorCodesProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n*tensorflow/core/protobuf/error_codes.proto\x12\x10tensorflow.error*\x84\x03\n\x04Code\x12\x06\n\x02OK\x10\x00\x12\r\n\tCANCELLED\x10\x01\x12\x0b\n\x07UNKNOWN\x10\x02\x12\x14\n\x10INVALID_ARGUMENT\x10\x03\x12\x15\n\x11DEADLINE_EXCEEDED\x10\x04\x12\r\n\tNOT_FOUND\x10\x05\x12\x12\n\x0eALREADY_EXISTS\x10\x06\x12\x15\n\x11PERMISSION_DENIED\x10\x07\x12\x13\n\x0fUNAUTHENTICATED\x10\x10\x12\x16\n\x12RESOURCE_EXHAUSTED\x10\x08\x12\x17\n\x13FAILED_PRECONDITION\x10\t\x12\x0b\n\x07ABORTED\x10\n\x12\x10\n\x0cOUT_OF_RANGE\x10\x0b\x12\x11\n\rUNIMPLEMENTED\x10\x0c\x12\x0c\n\x08INTERNAL\x10\r\x12\x0f\n\x0bUNAVAILABLE\x10\x0e\x12\r\n\tDATA_LOSS\x10\x0f\x12K\nGDO_NOT_USE_RESERVED_FOR_FUTURE_EXPANSION_USE_DEFAULT_IN_SWITCH_INSTEAD_\x10\x14B\x88\x01\n\x18org.tensorflow.frameworkB\x10ErrorCodesProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01b\x06proto3')
_CODE = _descriptor.EnumDescriptor(name='Code', full_name='tensorflow.error.Code', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='OK', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='CANCELLED', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='UNKNOWN', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='INVALID_ARGUMENT', index=3, number=3, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='DEADLINE_EXCEEDED', index=4, number=4, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='NOT_FOUND', index=5, number=5, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='ALREADY_EXISTS', index=6, number=6, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='PERMISSION_DENIED', index=7, number=7, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='UNAUTHENTICATED', index=8, number=16, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='RESOURCE_EXHAUSTED', index=9, number=8, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='FAILED_PRECONDITION', index=10, number=9, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='ABORTED', index=11, number=10, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='OUT_OF_RANGE', index=12, number=11, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='UNIMPLEMENTED', index=13, number=12, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='INTERNAL', index=14, number=13, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='UNAVAILABLE', index=15, number=14, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='DATA_LOSS', index=16, number=15, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='DO_NOT_USE_RESERVED_FOR_FUTURE_EXPANSION_USE_DEFAULT_IN_SWITCH_INSTEAD_', index=17, number=20, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=65, serialized_end=453)
_sym_db.RegisterEnumDescriptor(_CODE)
Code = enum_type_wrapper.EnumTypeWrapper(_CODE)
OK = 0
CANCELLED = 1
UNKNOWN = 2
INVALID_ARGUMENT = 3
DEADLINE_EXCEEDED = 4
NOT_FOUND = 5
ALREADY_EXISTS = 6
PERMISSION_DENIED = 7
UNAUTHENTICATED = 16
RESOURCE_EXHAUSTED = 8
FAILED_PRECONDITION = 9
ABORTED = 10
OUT_OF_RANGE = 11
UNIMPLEMENTED = 12
INTERNAL = 13
UNAVAILABLE = 14
DATA_LOSS = 15
DO_NOT_USE_RESERVED_FOR_FUTURE_EXPANSION_USE_DEFAULT_IN_SWITCH_INSTEAD_ = 20
DESCRIPTOR.enum_types_by_name['Code'] = _CODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
DESCRIPTOR._options = None



from __future__ import annotations
import os
import io as _io
import stat
import errno as _errno
from typing import List, Tuple, Optional, cast as _cast

from . import file_system
from . import file_system_helper
from . import file_statistics
from . import errors
from . import core
from . import std

# 128KB of copy buffer
kPosixCopyFileBufferSize = 128 * 1024

# pread() based random-access
class PosixRandomAccessFile(file_system.RandomAccessFile):
  def __init__(self, fname, fd):
    self._filename = core.string_view(fname).bytes()
    self._fd = fd
    self._closed = False

  def __repr__(self):
    return (f"PosixRandomAccessFile(fd={self._fd!r}, filename={self._filename!r}"
            + (", closed" if self._closed else "") + ")")

  def name(self) -> Tuple[errors.Status, bytes]:
    return errors.Status.OK(), self._filename

  def close(self) -> errors.Status:
    closed, self._closed = self._closed, True
    if not closed:
      try:
        os.close(self._fd)
      except IOError as e:
        return errors.IOError(self._filename, e.errno)
    return errors.Status.OK()

  def read(self, offset: int, n: int) -> Tuple[errors.Status, core.StringPiece]:
    fd = self._fd
    s = errors.Status.OK()
    dst = b''
    while n > 0 and s.ok():
      # Some platforms, notably macs, throw EINVAL if pread is asked to read
      # more than fits in a 32-bit integer.
      if n > std.INT32_MAX:
        requested_read_length = std.INT32_MAX
      else:
        requested_read_length = n
      errno = None
      try:
        ret = os.pread(fd, requested_read_length, offset)
        r = len(ret)
        if r < requested_read_length:
          r = 0
      except IOError as e:
        ret = b""
        r = -1
        errno = e.errno
      if r > 0:
        dst += ret
        n -= r
        offset += r
      elif r == 0:
        s = errors.OutOfRange("Read less bytes than requested")
      elif errno == _errno.EINTR or errno == _errno.EAGAIN:
        # Retry
        pass
      else:
        assert errno is not None
        s = errors.IOError(self._filename, errno)
    result = core.StringPiece(dst)
    return s, result

class PosixWritableFile(file_system.WritableFile):
  def __init__(self, fname, file: _io.BufferedWriter):
    self._filename = core.string_view(fname).bytes()
    self._file = file

  def __del__(self):
    # if (file_ != nullptr) {
    if self._file is not None:
      # // Ignoring any potential errors
      # fclose(file_);
      self._file.close()
    # }

  def __repr__(self):
    return f"PosixWritableFile(file={self._file!r}, filename={self._filename!r})"

  def append(self, data) -> errors.Status:
    data = core.string_view(data)
    r = self._file.write(data.bytes())
    if r != data.size():
      import ctypes
      errno = ctypes.get_errno()
      return errors.IOError(self._filename, errno)
    return errors.Status.OK()

  def name(self) -> Tuple[errors.Status, bytes]:
    return errors.Status.OK(), self._filename

  def close(self) -> errors.Status:
    if self._file is None:
      return errors.IOError(self._filename, _errno.EBADF)
    self._file.close()
    self._file = None
    return errors.Status.OK()

  def flush(self) -> errors.Status:
    try:
      self._file.flush()
      return errors.Status.OK()
    except IOError as e:
      return errors.IOError(self._filename, e.errno)

  def sync(self) -> errors.Status:
    try:
      self._file.flush()
      return errors.Status.OK()
    except IOError as e:
      return errors.IOError(self._filename, e.errno)

  def tell(self) -> Tuple[errors.Status, int]:
    try:
      return errors.Status.OK(), self._file.tell()
    except IOError as e:
      return errors.IOError(self._filename, e.errno), -1

class PosixFileSystem(file_system.FileSystem):
  def new_random_access_file(self, name) -> Tuple[errors.Status, Optional[file_system.RandomAccessFile]]:
    fname = self.translate_name(name)
    try:
      errno = None
      fd = os.open(fname, os.O_RDONLY)
    except IOError as e:
      errno = e.errno
      fd = -1
    if fd < 0:
      return errors.IOError(fname, errno), None
    else:
      result = PosixRandomAccessFile(fname, fd)
      return errors.Status.OK(), result

  def new_writable_file(self, name) -> Tuple[errors.Status, Optional[file_system.WritableFile]]:
    fname = self.translate_name(name)
    try:
      file = open(fname, "wb")
      result = PosixWritableFile(fname, _cast(_io.BufferedWriter, file))
      return errors.Status.OK(), result
    except IOError as e:
      return errors.IOError(fname, e.errno), None

  def rename_file(self, src, target) -> errors.Status:
    try:
      os.rename(self.translate_name(src), self.translate_name(target))
      return errors.Status.OK()
    except IOError as e:
      return errors.IOError(src, e.errno)

  def delete_file(self, name) -> errors.Status:
    try:
      os.unlink(self.translate_name(name))
      return errors.Status.OK()
    except IOError as e:
      return errors.IOError(name, e.errno)

  def file_exists(self, fname) -> errors.Status:
    if os.access(self.translate_name(fname), os.F_OK):
      return errors.Status.OK()
    return errors.NotFound(fname, " not found")

  def get_matching_paths(self, pattern) -> Tuple[errors.Status, Optional[List]]:
    from . import env # import here due to circular import
    return file_system_helper.get_matching_paths(self, env.Env.default(), pattern)

  def stat(self, name, stats: file_statistics.FileStatistics) -> errors.Status:
    fname = self.translate_name(name)
    try:
      sbuf = os.stat(fname)
    except IOError as e:
      return errors.IOError(fname, e.errno)
    stats.length = sbuf.st_size
    # stats->mtime_nsec = sbuf.st_mtime * 1e9;
    stats.mtime_nsec = sbuf.st_mtime_ns
    stats.is_directory = stat.S_ISDIR(sbuf.st_mode)
    return errors.Status.OK()

  def get_file_size(self, name) -> Tuple[errors.Status, int]:
    fname = self.translate_name(name)
    try:
      sbuf = os.stat(fname)
    except IOError as e:
      return errors.IOError(fname, e.errno), 0
    return errors.Status.OK(), sbuf.st_size

  def get_children(self, dir) -> Tuple[errors.Status, List[bytes]]:
    fname = self.translate_name(dir)
    result = []
    try:
      for d in os.scandir(fname):
        entry: os.DirEntry = d
        basename = core.string_view(entry.name).bytes()
        if basename not in [b".", b".."]:
          result.append(basename)
    except IOError as e:
      return errors.IOError(fname, e.errno), result
    return errors.Status.OK(), result










'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/protobuf/saver.proto', package='tensorflow', syntax='proto3', serialized_options=b'\n\x13org.tensorflow.utilB\x0bSaverProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n$tensorflow/core/protobuf/saver.proto\x12\ntensorflow"\x9e\x02\n\x08SaverDef\x12\x1c\n\x14filename_tensor_name\x18\x01 \x01(\t\x12\x18\n\x10save_tensor_name\x18\x02 \x01(\t\x12\x17\n\x0frestore_op_name\x18\x03 \x01(\t\x12\x13\n\x0bmax_to_keep\x18\x04 \x01(\x05\x12\x0f\n\x07sharded\x18\x05 \x01(\x08\x12%\n\x1dkeep_checkpoint_every_n_hours\x18\x06 \x01(\x02\x12=\n\x07version\x18\x07 \x01(\x0e2,.tensorflow.SaverDef.CheckpointFormatVersion"5\n\x17CheckpointFormatVersion\x12\n\n\x06LEGACY\x10\x00\x12\x06\n\x02V1\x10\x01\x12\x06\n\x02V2\x10\x02B~\n\x13org.tensorflow.utilB\x0bSaverProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01b\x06proto3')
_SAVERDEF_CHECKPOINTFORMATVERSION = _descriptor.EnumDescriptor(name='CheckpointFormatVersion', full_name='tensorflow.SaverDef.CheckpointFormatVersion', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='LEGACY', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='V1', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='V2', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=286, serialized_end=339)
_sym_db.RegisterEnumDescriptor(_SAVERDEF_CHECKPOINTFORMATVERSION)
_SAVERDEF = _descriptor.Descriptor(name='SaverDef', full_name='tensorflow.SaverDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='filename_tensor_name', full_name='tensorflow.SaverDef.filename_tensor_name', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='save_tensor_name', full_name='tensorflow.SaverDef.save_tensor_name', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='restore_op_name', full_name='tensorflow.SaverDef.restore_op_name', index=2, number=3, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='max_to_keep', full_name='tensorflow.SaverDef.max_to_keep', index=3, number=4, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='sharded', full_name='tensorflow.SaverDef.sharded', index=4, number=5, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='keep_checkpoint_every_n_hours', full_name='tensorflow.SaverDef.keep_checkpoint_every_n_hours', index=5, number=6, type=2, cpp_type=6, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='version', full_name='tensorflow.SaverDef.version', index=6, number=7, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[_SAVERDEF_CHECKPOINTFORMATVERSION], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=53, serialized_end=339)
_SAVERDEF.fields_by_name['version'].enum_type = _SAVERDEF_CHECKPOINTFORMATVERSION
_SAVERDEF_CHECKPOINTFORMATVERSION.containing_type = _SAVERDEF
DESCRIPTOR.message_types_by_name['SaverDef'] = _SAVERDEF
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
SaverDef = _reflection.GeneratedProtocolMessageType('SaverDef', (_message.Message,), {'DESCRIPTOR': _SAVERDEF, '__module__': 'tensorflow.core.protobuf.saver_pb2'})
_sym_db.RegisterMessage(SaverDef)
DESCRIPTOR._options = None



from . import strings
from . import core

def meta_filename(prefix) -> bytes:
  prefix = core.string_view(prefix)
  return strings.printf(b"%.*s.index", prefix.size(), prefix.bytes())

def data_filename(prefix, shard_id: int, num_shards: int) -> bytes:
  assert num_shards > 0
  assert 0 <= shard_id < num_shards
  prefix = core.string_view(prefix)
  return strings.printf(b"%.*s.data-%05d-of-%05d", prefix.size(), prefix.bytes(), shard_id, num_shards)


# /* Copyright 2016 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================*/
#
# #ifndef TENSORFLOW_CORE_LIB_IO_RANDOM_INPUTSTREAM_H_
# #define TENSORFLOW_CORE_LIB_IO_RANDOM_INPUTSTREAM_H_
#
# #include "tensorflow/core/lib/io/inputstream_interface.h"
# #include "tensorflow/core/platform/cord.h"
# #include "tensorflow/core/platform/file_system.h"
#
# namespace tensorflow {
# namespace io {
#
# // Wraps a RandomAccessFile in an InputStreamInterface. A given instance of
# // RandomAccessInputStream is NOT safe for concurrent use by multiple threads.
# class RandomAccessInputStream : public InputStreamInterface {
#  public:
#   // Does not take ownership of 'file' unless owns_file is set to true. 'file'
#   // must outlive *this.
#   RandomAccessInputStream(RandomAccessFile* file, bool owns_file = false);
#
#   ~RandomAccessInputStream();
#
#   Status ReadNBytes(int64 bytes_to_read, tstring* result) override;
#
# #if defined(TF_CORD_SUPPORT)
#   Status ReadNBytes(int64 bytes_to_read, absl::Cord* result) override;
# #endif
#
#   Status SkipNBytes(int64 bytes_to_skip) override;
#
#   int64 Tell() const override;
#
#   Status Seek(int64 position) {
#     pos_ = position;
#     return Status::OK();
#   }
#
#   Status Reset() override { return Seek(0); }
#
#  private:
#   RandomAccessFile* file_;  // Not owned.
#   int64 pos_ = 0;           // Tracks where we are in the file.
#   bool owns_file_ = false;
# };
#
# }  // namespace io
# }  // namespace tensorflow
#
# #endif  // TENSORFLOW_CORE_LIB_IO_RANDOM_INPUTSTREAM_H_
from typing import Tuple

from tensorflow_checkpoint_reader import errors

# /* Copyright 2016 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================*/
#
# #include "tensorflow/core/lib/io/random_inputstream.h"
# #include <memory>
#
# namespace tensorflow {
# namespace io {
#
# RandomAccessInputStream::RandomAccessInputStream(RandomAccessFile* file,
#                                                  bool owns_file)
#     : file_(file), owns_file_(owns_file) {}
#
# RandomAccessInputStream::~RandomAccessInputStream() {
#   if (owns_file_) {
#     delete file_;
#   }
# }
#
# Status RandomAccessInputStream::ReadNBytes(int64 bytes_to_read,
#                                            tstring* result) {
#   if (bytes_to_read < 0) {
#     return errors::InvalidArgument("Cannot read negative number of bytes");
#   }
#   result->clear();
#   result->resize_uninitialized(bytes_to_read);
#   char* result_buffer = &(*result)[0];
#   StringPiece data;
#   Status s = file_->Read(pos_, bytes_to_read, &data, result_buffer);
#   if (data.data() != result_buffer) {
#     memmove(result_buffer, data.data(), data.size());
#   }
#   result->resize(data.size());
#   if (s.ok() || errors::IsOutOfRange(s)) {
#     pos_ += data.size();
#   }
#   return s;
# }
#
# #if defined(TF_CORD_SUPPORT)
# Status RandomAccessInputStream::ReadNBytes(int64 bytes_to_read,
#                                            absl::Cord* result) {
#   if (bytes_to_read < 0) {
#     return errors::InvalidArgument("Cannot read negative number of bytes");
#   }
#   int64 current_size = result->size();
#   Status s = file_->Read(pos_, bytes_to_read, result);
#   if (s.ok() || errors::IsOutOfRange(s)) {
#     pos_ += result->size() - current_size;
#   }
#   return s;
# }
# #endif
#
# // To limit memory usage, the default implementation of SkipNBytes() only reads
# // 8MB at a time.
# static constexpr int64 kMaxSkipSize = 8 * 1024 * 1024;
#
# Status RandomAccessInputStream::SkipNBytes(int64 bytes_to_skip) {
#   if (bytes_to_skip < 0) {
#     return errors::InvalidArgument("Can't skip a negative number of bytes");
#   }
#   std::unique_ptr<char[]> scratch(new char[kMaxSkipSize]);
#   // Try to read 1 bytes first, if we could complete the read then EOF is
#   // not reached yet and we could return.
#   if (bytes_to_skip > 0) {
#     StringPiece data;
#     Status s = file_->Read(pos_ + bytes_to_skip - 1, 1, &data, scratch.get());
#     if ((s.ok() || errors::IsOutOfRange(s)) && data.size() == 1) {
#       pos_ += bytes_to_skip;
#       return Status::OK();
#     }
#   }
#   // Read kDefaultSkipSize at a time till bytes_to_skip.
#   while (bytes_to_skip > 0) {
#     int64 bytes_to_read = std::min<int64>(kMaxSkipSize, bytes_to_skip);
#     StringPiece data;
#     Status s = file_->Read(pos_, bytes_to_read, &data, scratch.get());
#     if (s.ok() || errors::IsOutOfRange(s)) {
#       pos_ += data.size();
#     } else {
#       return s;
#     }
#     if (data.size() < static_cast<size_t>(bytes_to_read)) {
#       return errors::OutOfRange("reached end of file");
#     }
#     bytes_to_skip -= bytes_to_read;
#   }
#   return Status::OK();
# }
#
# int64 RandomAccessInputStream::Tell() const { return pos_; }
#
# }  // namespace io
# }  // namespace tensorflow

from . import inputstream_interface
from ..... import file_system

from typing import Tuple, Optional


class RandomAccessInputStream(inputstream_interface.InputStreamInterface):
  """Wraps a RandomAccessFile in an InputStreamInterface. A given instance of
  RandomAccessInputStream is NOT safe for concurrent use by multiple threads."""

  #   // Does not take ownership of 'file' unless owns_file is set to true. 'file'
  #   // must outlive *this.
  #   RandomAccessInputStream(RandomAccessFile* file, bool owns_file = false);
  def __init__(self, file: file_system.RandomAccessFile, owns_file: bool = False):
    """Does not take ownership of 'file' unless owns_file is set to true. 'file'
    must outlive self."""
    self.file_ = file
    self.owns_file_ = owns_file
    self.pos_ = 0

  def read_n_bytes(self, bytes_to_read: int) -> Tuple[errors.Status, Optional[bytearray]]:
    #   if (bytes_to_read < 0) {
    #     return errors::InvalidArgument("Cannot read negative number of bytes");
    #   }
    if bytes_to_read < 0:
      return errors.InvalidArgument("Cannot read negative number of bytes"), None
    #   result->clear();
    #   result->resize_uninitialized(bytes_to_read);
    #   char* result_buffer = &(*result)[0];
    #   StringPiece data;
    #   Status s = file_->Read(pos_, bytes_to_read, &data, result_buffer);
    s, data = self.file_.read(self.pos_, bytes_to_read)
    #   if (data.data() != result_buffer) {
    #     memmove(result_buffer, data.data(), data.size());
    #   }
    #   result->resize(data.size());
    #   if (s.ok() || errors::IsOutOfRange(s)) {
    if s.ok() or errors.is_out_of_range(s):
      #   pos_ += data.size();
      self.pos_ += data.size()
    #   }
    #   return s;
    return s, data.slice()

  def skip_n_bytes(self, bytes_to_skip: int) -> errors.Status:
    raise NotImplementedError

  # int64 RandomAccessInputStream::Tell() const { return pos_; }
  def tell(self) -> int:
    return self.pos_

  def reset(self):
    raise NotImplementedError

